-- FINVSQRT
-- ニュートン法による平方根の逆数

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity FINVSQRT_n is

	port(SRC            : in std_logic_vector(31 downto 0);
        CLK            : in std_logic;
        FINVSQRT_START : in std_logic;
        FINVSQRT_END   : out std_logic;
	     OUTPUT         : out std_logic_vector(31 downto 0));

end FINVSQRT_n;

architecture STRUCTURE of FINVSQRT_n is

component FINVSQRT_repeat_1 is
	port(SRC    : in std_logic_vector(31 downto 0);
        EXP    : out std_logic_vector(7 downto 0);
        Zero   : out std_logic;
	     Xhout  : out std_logic_vector(12 downto 0);
        Xlout  : out std_logic_vector(11 downto 0);
        Aout   : out std_logic_vector(22 downto 0);
	     AXout  : out std_logic_vector(29 downto 0));
end component;

component FINVSQRT_repeat_2 is
	port(EXPin          : in std_logic_vector(7 downto 0);
        Zeroin         : in std_logic;
	     Xhin           : in std_logic_vector(12 downto 0);
        Xlin           : in std_logic_vector(11 downto 0);
        Ain            : in std_logic_vector(22 downto 0);
	     AXin           : in std_logic_vector(29 downto 0);
        EXPout         : out std_logic_vector(7 downto 0);
        Zeroout        : out std_logic;
	     Xhout          : out std_logic_vector(12 downto 0);
        Xlout          : out std_logic_vector(11 downto 0);
        Aout           : out std_logic_vector(22 downto 0);
	     ThreeMinusAXX  : out std_logic_vector(27 downto 0));
end component;

component FINVSQRT_repeat_3 is
	port(EXPin          : in std_logic_vector(7 downto 0);
        Zeroin         : in std_logic;
	     Xhin           : in std_logic_vector(12 downto 0);
        Xlin           : in std_logic_vector(11 downto 0);
        Ain            : in std_logic_vector(22 downto 0);
	     ThreeMinusAXX  : in std_logic_vector(27 downto 0);
        EXPout         : out std_logic_vector(7 downto 0);
        Zeroout        : out std_logic;
        Aout           : out std_logic_vector(22 downto 0);
        nextX          : out std_logic_vector(24 downto 0));
end component;

component FINVSQRT_repeat_4 is
	port(EXPin    : in std_logic_vector(7 downto 0);
        Zeroin   : in std_logic;
        Ain      : in std_logic_vector(22 downto 0);
	     nextX    : in std_logic_vector(24 downto 0);
	     EXPout   : out std_logic_vector(7 downto 0);
        Zeroout  : out std_logic;
	     Xhout    : out std_logic_vector(12 downto 0);
        Xlout    : out std_logic_vector(11 downto 0);
	     AXout    : out std_logic_vector(29 downto 0));
end component;

component FINVSQRT_repeat_5 is
	port(EXPin          : in std_logic_vector(7 downto 0);
        Zeroin         : in std_logic;
	     Xhin           : in std_logic_vector(12 downto 0);
        Xlin           : in std_logic_vector(11 downto 0);
	     AXin           : in std_logic_vector(29 downto 0);
        EXPout         : out std_logic_vector(7 downto 0);
        Zeroout        : out std_logic;
	     Xhout          : out std_logic_vector(12 downto 0);
             Xlout          : out std_logic_vector(11 downto 0);
	     ThreeMinusAXX  : out std_logic_vector(27 downto 0));
end component;

component FINVSQRT_repeat_6 is
	port(EXP            : in std_logic_vector(7 downto 0);
        Zero           : in std_logic;
	     Xhin           : in std_logic_vector(12 downto 0);
        Xlin           : in std_logic_vector(11 downto 0);
	     ThreeMinusAXX  : in std_logic_vector(27 downto 0);
        OUTPUT         : out std_logic_vector(31 downto 0));
end component;


-- ↓非パイプラインまたは数値テスト用
signal EXP12 : std_logic_vector(7 downto 0);
signal EXP23 : std_logic_vector(7 downto 0);
signal EXP34 : std_logic_vector(7 downto 0);
signal EXP45 : std_logic_vector(7 downto 0);
signal EXP56 : std_logic_vector(7 downto 0);
signal Zero12 : std_logic;
signal Zero23 : std_logic;
signal Zero34 : std_logic;
signal Zero45 : std_logic;
signal Zero56 : std_logic;
signal Xh12 : std_logic_vector(12 downto 0);
signal Xh23 : std_logic_vector(12 downto 0);
signal Xh45 : std_logic_vector(12 downto 0);
signal Xh56 : std_logic_vector(12 downto 0);
signal Xl12 : std_logic_vector(11 downto 0);
signal Xl23 : std_logic_vector(11 downto 0);
signal Xl45 : std_logic_vector(11 downto 0);
signal Xl56 : std_logic_vector(11 downto 0);
signal A12  : std_logic_vector(22 downto 0);
signal A23  : std_logic_vector(22 downto 0);
signal A34  : std_logic_vector(22 downto 0);
signal AX12 : std_logic_vector(29 downto 0);
signal AX45 : std_logic_vector(29 downto 0);
signal ThreeMinusAXX23 : std_logic_vector(27 downto 0);
signal ThreeMinusAXX56 : std_logic_vector(27 downto 0);
signal nextX34 : std_logic_vector(24 downto 0);
signal CountClock : std_logic_vector(3 downto 0) := "0000";
signal CountClock2 : std_logic_vector(3 downto 0) := "0000";
-- ↑非パイプライン用

---- ↓パイプライン用
--signal EXP12 : std_logic_vector(7 downto 0);
--signal EXP23 : std_logic_vector(7 downto 0);
--signal EXP34 : std_logic_vector(7 downto 0);
--signal EXP45 : std_logic_vector(7 downto 0);
--signal EXP56 : std_logic_vector(7 downto 0);
--signal Zero12 : std_logic;
--signal Zero23 : std_logic;
--signal Zero34 : std_logic;
--signal Zero45 : std_logic;
--signal Zero56 : std_logic;
--signal Xh12 : std_logic_vector(12 downto 0);
--signal Xh23 : std_logic_vector(12 downto 0);
--signal Xh45 : std_logic_vector(12 downto 0);
--signal Xh56 : std_logic_vector(12 downto 0);
--signal Xl12 : std_logic_vector(11 downto 0);
--signal Xl23 : std_logic_vector(11 downto 0);
--signal Xl45 : std_logic_vector(11 downto 0);
--signal Xl56 : std_logic_vector(11 downto 0);
--signal A12  : std_logic_vector(22 downto 0);
--signal A23  : std_logic_vector(22 downto 0);
--signal A34  : std_logic_vector(22 downto 0);
--signal AX12 : std_logic_vector(29 downto 0);
--signal AX45 : std_logic_vector(29 downto 0);
--signal ThreeMinusAXX23 : std_logic_vector(27 downto 0);
--signal ThreeMinusAXX56 : std_logic_vector(27 downto 0);
--signal nextX34 : std_logic_vector(24 downto 0);
--signal EXP12in : std_logic_vector(7 downto 0);
--signal EXP23in : std_logic_vector(7 downto 0);
--signal EXP34in : std_logic_vector(7 downto 0);
--signal EXP45in : std_logic_vector(7 downto 0);
--signal EXP56in : std_logic_vector(7 downto 0);
--signal Zero12in : std_logic;
--signal Zero23in : std_logic;
--signal Zero34in : std_logic;
--signal Zero45in : std_logic;
--signal Zero56in : std_logic;
--signal Xh12in : std_logic_vector(12 downto 0);
--signal Xh23in : std_logic_vector(12 downto 0);
--signal Xh45in : std_logic_vector(12 downto 0);
--signal Xh56in : std_logic_vector(12 downto 0);
--signal Xl12in : std_logic_vector(11 downto 0);
--signal Xl23in : std_logic_vector(11 downto 0);
--signal Xl45in : std_logic_vector(11 downto 0);
--signal Xl56in : std_logic_vector(11 downto 0);
--signal A12in  : std_logic_vector(22 downto 0);
--signal A23in  : std_logic_vector(22 downto 0);
--signal A34in  : std_logic_vector(22 downto 0);
--signal AX12in : std_logic_vector(29 downto 0);
--signal AX45in : std_logic_vector(29 downto 0);
--signal ThreeMinusAXX23in : std_logic_vector(27 downto 0);
--signal ThreeMinusAXX56in : std_logic_vector(27 downto 0);
--signal nextX34in : std_logic_vector(24 downto 0);
--signal CountClock : std_logic_vector(4 downto 0);
--signal CountClock2 : std_logic_vector(4 downto 0);
---- ↑パイプライン用


begin -- STRUCTURE
   --　↓非パイプライン実行用
     FINVSQRT_repeat1_1 : FINVSQRT_repeat_1 port map (SRC,EXP12,Zero12,Xh12,Xl12,A12,AX12);
     FINVSQRT_repeat2_1 : FINVSQRT_repeat_2 port map (EXP12,Zero12,Xh12,Xl12,A12,AX12,EXP23,Zero23,Xh23,Xl23,A23,ThreeMinusAXX23);
     FINVSQRT_repeat3_1 : FINVSQRT_repeat_3 port map (EXP23,Zero23,Xh23,Xl23,A23,ThreeMinusAXX23,EXP34,Zero34,A34,nextX34);
     FINVSQRT_repeat4_1 : FINVSQRT_repeat_4 port map (EXP34,Zero34,A34,nextX34,EXP45,Zero45,Xh45,Xl45,AX45);
     FINVSQRT_repeat5_1 : FINVSQRT_repeat_5 port map (EXP45,Zero45,Xh45,Xl45,AX45,EXP56,Zero56,Xh56,Xl56,ThreeMinusAXX56);
     FINVSQRT_repeat6_1 : FINVSQRT_repeat_6 port map (EXP56,Zero56,Xh56,Xl56,ThreeMinusAXX56,OUTPUT);

   FINVSQRT : process(CLK)
   begin
      if CLK'event and CLK = '1' then
            CountClock <= CountClock2(2 downto 0) & FINVSQRT_START;
         --FADSB_END <= FADSB_START;
      end if;
      if CLK'event and CLK = '0' then
            CountClock2 <= CountClock;
            FINVSQRT_END <= CountClock(3);
      end if;
   end process FINVSQRT;



   --　↑非パイプライン実行用


----   --　↓パイプライン用
--     FINVSQRT_repeat1_1 : FINVSQRT_repeat_1 port map (SRC,EXP12,Zero12,Xh12,Xl12,A12,AX12);
--     FINVSQRT_repeat2_1 : FINVSQRT_repeat_2 port map (EXP12in,Zero12in,Xh12in,Xl12in,A12in,AX12in,EXP23,Zero23,Xh23,Xl23,A23,ThreeMinusAXX23);
--     FINVSQRT_repeat3_1 : FINVSQRT_repeat_3 port map (EXP23in,Zero23in,Xh23in,Xl23in,A23in,ThreeMinusAXX23in,EXP34,Zero34,A34,nextX34);
--     FINVSQRT_repeat4_1 : FINVSQRT_repeat_4 port map (EXP34in,Zero34in,A34in,nextX34in,EXP45,Zero45,Xh45,Xl45,AX45);
--     FINVSQRT_repeat5_1 : FINVSQRT_repeat_5 port map (EXP45in,Zero45in,Xh45in,Xl45in,AX45in,EXP56,Zero56,Xh56,Xl56,ThreeMinusAXX56);
--     FINVSQRT_repeat6_1 : FINVSQRT_repeat_6 port map (EXP56in,Zero56in,Xh56in,Xl56in,ThreeMinusAXX56in,OUTPUT);
--
--   FINVSQRT : process(CLK)
--   begin
--      if CLK'event and CLK = '1' then
--            EXP12in <= EXP12;
--            Zero12in <= Zero12;
--            Xh12in <= Xh12;
--            Xl12in <= Xl12;
--            A12in <= A12;
--            AX12in <= AX12;
--            EXP23in <= EXP23;
--            Zero23in <= Zero23;
--            Xh23in <= Xh23;
--            Xl23in <= Xl23;
--            A23in <= A23;
--            ThreeMinusAXX23in <= ThreeMinusAXX23;
--            EXP34in <= EXP34;
--            Zero34in <= Zero34;
--            A34in <= A34;
--            nextX34in <= nextX34;
--            EXP45in <= EXP45;
--            Zero45in <= Zero45;
--            Xh45in <= Xh45;
--            Xl45in <= Xl45;
--            AX45in <= AX45;
--            EXP56in <= EXP56;
--            Zero56in <= Zero56;
--            Xh56in <= Xh56;
--            Xl56in <= Xl56;
--            ThreeMinusAXX56in <= ThreeMinusAXX56;
--
--            CountClock <= CountClock2(3 downto 0) & FINVSQRT_START;
--         --FADSB_END <= FADSB_START;
--      end if;
--      if CLK'event and CLK = '0' then
--            CountClock2 <= CountClock;
--            FINVSQRT_END <= CountClock(4);
--      end if;
--   end process FINVSQRT;
--   --　↑パイプライン用


end STRUCTURE;
