-- FINVSQRTの繰り返し計算部分
-- X(n+1) = (3 - A*X(n)*X(n))*X(n)/2を行う
-- 

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity FINVSQRT_repeat is

	port(X      : in std_logic_vector(24 downto 0);    --MSB含む
	     A      : in std_logic_vector(24 downto 0);
	     nextX : out std_logic_vector(24 downto 0));

end FINVSQRT_repeat;

architecture STRUCTURE of FINVSQRT_repeat is

signal AX : std_logic_vector(49 downto 0);         -- A*X(n)
signal AXX : std_logic_vector(54 downto 0);        -- A*X(n)*X(n)
signal AXX_buf : std_logic_vector(54 downto 0);
signal ThreeMinusAXX : std_logic_vector(27 downto 0);
signal nextX_buf : std_logic_vector(52 downto 0);

signal Ah : std_logic_vector(12 downto 0);
signal Al : std_logic_vector(11 downto 0);
signal Xh : std_logic_vector(12 downto 0);
signal Xl : std_logic_vector(11 downto 0);
signal AXhh : std_logic_vector(25 downto 0);
signal AXhl : std_logic_vector(24 downto 0);
signal AXlh : std_logic_vector(24 downto 0);
signal AXll : std_logic_vector(23 downto 0);
signal nextXhh : std_logic_vector(26 downto 0);
signal nextXhl : std_logic_vector(26 downto 0);
signal nextXlh : std_logic_vector(25 downto 0);
signal nextXll : std_logic_vector(25 downto 0);

begin -- STRUCTURE


Ah <= A(24 downto 12);
Al <= A(11 downto 0);
Xh <= X(24 downto 12);
Xl <= X(11 downto 0);
AXhh <= Ah * Xh;
AXhl <= Ah * Xl;
AXlh <= Al * Xh;
AXll <= Al * Xl;
AX <= (AXhh & AXll) + ("000000000000" & (('0' & AXhl) + ('0' & AXlh))  & "000000000000");

--AXX_buf <= X * AX;
--AXX_buf <= (X * AX(49 downto 14)) & "00000000000000";
AXX_buf <= (X * AX(49 downto 20));




AXX <= AXX_buf(53 downto 0) & '0' when AXX_buf(54 downto 53) = "00" else AXX_buf;
-- AXX_bufの頭は"001"か"01"か'1'（掛け算により、先頭の'1'が最大で1つ右にずれる）

ThreeMinusAXX <= "1100000000000000000000000000" - ('0' & AXX(54 downto 28));--("0" & AXX(37 downto 13));


nextXhh <= ThreeMinusAXX(27 downto 14) * Xh;
nextXhl <= ThreeMinusAXX(13 downto  0) * Xh;
nextXlh <= ThreeMinusAXX(27 downto 14) * Xl;
nextXll <= ThreeMinusAXX(13 downto  0) * Xl;
nextX_buf <= (nextXhh & nextXll) + (("00000000000000" & nextXhl & "000000000000") + ("0000000000000" & nextXlh & "00000000000000"));

nextX <= nextX_buf(52 downto 28) when nextX_buf(52) = '1' else
         nextX_buf(51 downto 27);


end STRUCTURE;
