-------------------
-- USB version 2
-------------------

---- Description -----------
-- entity USB
-- architecture ARC of USB 
--
-- USB controller
-- interface signals
--        CLK : in std_logic;                          -- clock 50MHz
--        RESET : in std_logic;                        -- reset signal
--        RMODE : in std_logic;                        -- 1 when read
--        WMODE : in std_logic;                        -- 1 when write
--        RDATA : out std_logic_vector(7 downto 0);    -- read data
--        WDATA : in std_logic_vector(7 downto 0);     -- write data
--        VALID : out std_logic;                       -- 1 when read is O.K.
--        BUSY : out std_logic;
--  signals to USB
--        --RSTX : out std_logic;                        -- USB reset
--        RDX : out std_logic;                         -- 0 when read
--        WR : out std_logic;                          -- 1 when write
--        TXEX : in std_logic;                         -- 1 when buffer is full
--        RXFX : in std_logic;                         -- 1 when buffer is empty
--        USBDATA : inout std_logic_vector(7 downto 0));   -- USB data
--
----------------------------

---- Notes --------------------------------------------------------
-- 2006.12.04
--   ˽ľޤԥھ֤˶ᤤɡ
--   50MHzΥå
--
-- Version 1 εϿ
--   2006.10.26 
--   äȽ񤤤Ƥߤ롣
--   clockλϤɤ褦ͤ100MHzȻפʤΤ
--   2006.10.27
--   ǡϥߥ󥰤񤭴ץˤƤߤ롣
--   ˽񤭴ValidĤ롣
-------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity USB is
    port(
        CLK : in std_logic;                          -- clock 50MHz
        RESET : in std_logic;                        -- reset signal
        RMODE : in std_logic;                        -- 1 when read
        WMODE : in std_logic;                        -- 1 when write
        RDATA : out std_logic_vector(7 downto 0);    -- read data
        WDATA : in std_logic_vector(7 downto 0);     -- write data
        VALID : out std_logic;                       -- 1 when read is O.K.
        BUSY : out std_logic;
        -- to USB
        --RSTX : out std_logic;                        -- USB reset
        RDX : out std_logic;                         -- 0 when read
        WR : out std_logic;                          -- 1 when write
        TXEX : in std_logic;                         -- 1 when buffer is full
        RXFX : in std_logic;                         -- 1 when buffer is empty
        USBDATA : inout std_logic_vector(7 downto 0));   -- USB data

end USB;

architecture ARC of USB is
    signal STATE : std_logic_vector(3 downto 0);     -- state

begin
    USBDATA <= WDATA when WMODE = '1' else
                "ZZZZZZZZ";
    RDATA <= USBDATA;
--    RSTX <= '0';

    process(CLK,RESET)
    begin
        if RESET = '1' then
            -- port
            VALID <= '0';
            BUSY <= '0';
            -- to USB
--            RSTX <= '1';
            RDX <= '1';
            WR <= '0';
            -- state
            STATE <= "0000";
        elsif CLK'event and CLK = '1' then
            case STATE is
                when "0000" =>
                    if RMODE = '1' then        -- read cycle "0---"
                        if RXFX = '1' then
                            STATE <= "0000";
                        else
                            RDX <= '0';
                            BUSY <= '1';
                            VALID <= '0';
                            STATE <= "0001";
                        end if;
                    elsif WMODE = '1' then     -- write cycle "1---"
                        if TXEX = '1' then
                            STATE <= "0000";
                        else
                            WR <= '1';
                            BUSY <= '1';
                            VALID <= '0';
                            STATE <= "1001";
                        end if;
                    else
                        STATE <= "0000";
                    end if;
                -- read cycle
                when "0001" | "0100" =>
                    STATE <= STATE + 1;
                when "0010" =>
                    VALID <= '1';          -- can read
                    STATE <= STATE + 1;
                when "0011" =>
                    RDX <= '1';
                    VALID <= '0';
                    STATE <= STATE + 1;
                -- write cycle
                when "1001" | "1010" =>
                    STATE <= STATE + 1;
                when "1011" =>
                    WR <= '0';
                    STATE <= STATE + 1;
                when "1100" =>
                    VALID <= '1';           -- written
                    STATE <= STATE + 1;
                -- end cycle
                when "0101" | "1101" =>
                    VALID <= '0';
                    BUSY <= '0';
                    STATE <= "0000";
                -- others ( end cycle )
                when "0110" | "0111" | "1110" | "1111" =>
                    VALID <= '0';
                    BUSY <= '0';
                    STATE <= "0000";
                when others =>
                    VALID <= '0';
                    BUSY <= '0';
                    STATE <= "0000";
            end case;
        end if; -- CLK
    end process;
end ARC;

